import discord

from rizthme.models.threads import Player
from rizthme.models.mode import MODE


def init(client: "Client"):

    @client.command(["unloop"])
    async def unloop(message: discord.Message):
        """
        Setting up the normal mode to the music player in the guild.
        """
        guild: discord.Guild = message.guild or message.author.guild
        Player.get(guild).set_mode(MODE.NORMAL)



import re
from typing import Optional

import discord

from abc import ABC

from rizthme.exception import BadLinkError
from rizthme.models.musics import YTMusic, YTPlaylist, Playable


class AudioFactory(ABC):
    """
    Factory class for creating music objects.
    """

    client: Optional["Client"] = None

    dict_regex = {
        YTMusic: r"^https?:\/\/(www.)?(youtube.com|youtu.be)\/watch?(.*)$",
        YTPlaylist: r"^https?:\/\/(www.)?(youtube.com|youtu.be)\/playlist?(.*)$",
    }

    def __init__(self):
        raise NotImplementedError("This is an abstract class")

    @classmethod
    def create_playable(cls, message: discord.Message) -> Playable:
        """
        Create a music object based on the message.

        That will check by a regex if the message is a valid link.

        the @dict_regex while be used to have the regex for a specific type of music.
        """
        url = message.content.split(" ")[-1]
        channel = message.channel
        for PlayableClass, regex in cls.dict_regex.items():
            if re.match(regex, url):
                return PlayableClass(cls.client, url, channel)
        raise BadLinkError(f'your URL "{url}" is not a valid link.')



import asyncio
from typing import Union

import discord

from rizthme.models.threads import Player


def init(client: "Client"):

    @client.command(["play"])
    async def play(message: discord.Message) -> None:
        """
        plays music in the voice channel of the user who wrote the command.

        The command allows playing YouTube links and play audio without downloading the video.

        @play also allows to put a queue, the links called while the bot is already playing a sound.

        :param message: discord.Message, message sent by the user wanting to launch music
        """
        guild: discord.Guild = message.guild or message.author.guild
        # Retrieves the bot's VoiceClient, if there is already one present in the server.
        voice_client = discord.utils.get(client.voice_clients, guild=guild)
        if not voice_client:
            # Recover the voice channel of the message author.
            voice_channel: discord.VoiceChannel = message.author.voice.channel
            # if not, connect (by creating) to the server with a discord.VoiceClient
            voice_client: Union[discord.VoiceClient, discord.VoiceProtocol] = await voice_channel.connect(reconnect=False)
            # Set the new VoiceClient to the appropriate thread
            Player.set_voice_client(guild, voice_client)

        # Call Player,to add music
        Player.add_music(message)



from abc import ABC, abstractmethod
from typing import Callable

import discord


class Playable(ABC):
    """
    Abstract class for playable items.
    """

    @abstractmethod
    def is_valid(self, send_message: bool = False) -> bool:
        """
        Checks if the playable item is valid.
        """
        pass

    @abstractmethod
    def play(self, voice_client: discord.VoiceClient, after: Callable = None):
        """
        Plays the playable item.
        """
        pass

    @abstractmethod
    def stop(self, voice_client: discord.VoiceClient):
        """
        Stops the playable item.
        """
        pass

    @abstractmethod
    def get_audio_source(self):
        """
        Returns the audio source of the playable item.
        """
        pass

    @abstractmethod
    def get_duration(self) -> int:
        """
        :return: duration of the music in seconds
        """
        pass



import asyncio
import logging
from typing import Dict

import discord

from rizthme.models.threads import Player

timeout = 10

in_progress_schedulers: Dict[discord.VoiceChannel, asyncio.Task] = {}


def init(client: "Client"):

    @client.event
    async def on_voice_state_update(member: discord.Member, before: discord.VoiceState, after: discord.VoiceState):
        """
        This event is called when a member changes voice state.

        We try to check here, if the CLIENT is alone in the voice channel.
        If so, we try to shedule a task to set a timeout before rechecking if CLIENT is always alone in the voice channel.

        If a member join the CLIENT, that's stop the sheduler.

        :param member: Member concerning the voice state update
        :param before: VoiceState object before the change
        :param after: VoiceState object after the change
        """

        # if member don't have switched voice channel, do nothing
        if before.channel == after.channel:
            return

        # if member join the voice channel, stop the asyncio.Task
        if after.channel in in_progress_schedulers.keys():
            # cancel it
            in_progress_schedulers[after.channel].cancel()
            # delete it from the dict
            del in_progress_schedulers[after.channel]

        # get the voice client of the old voice channel member. if the member not being in the bot voice channel, do nothing
        voice_client: discord.VoiceClient = discord.utils.get(client.voice_clients, channel=before.channel)
        if voice_client is None:
            return

        # an async function to check if CLIENT always alone in the voice channel after timeout (in seconds)
        async def schedule():
            # wait for timeout seconds
            await asyncio.sleep(timeout)

            # if CLIENT is alone
            if check_alone_in_voice_channel(voice_client):
                # stop music and clear the queue
                p = Player.get(voice_client.guild)
                p.clear_queue()
                # disconnect from the voice channel
                dc = voice_client.disconnect()
                # delete this schedule from @in_progress_schedulers
                del in_progress_schedulers[voice_client.channel]
                await dc
                logging.info("Disconnected from voice channel. (By alone timeout task)")

        # Check if the member is in a voice channel
        if check_alone_in_voice_channel(voice_client):
            # Create a new asyncio.Task. schedule() will wait @timeout for recheck if CLIENT is alone
            task = asyncio.ensure_future(schedule())
            # add this task to @in_progress_schedulers
            in_progress_schedulers[voice_client.channel] = task


def check_alone_in_voice_channel(voice_client: discord.VoiceClient):
    """
    Check if the member is alone in the voice channel.
    :param voice_client: VoiceClient object
    :return: True if the member is alone, False otherwise
    """
    channel: discord.VoiceChannel = voice_client.channel
    if len(channel.members) == 1:
        return True
    return False



import discord

from rizthme.models.threads import Player


def init(client: "Client"):

    @client.command(["now", "nowplaying"])
    async def now(message: discord.Message):  # sourcery skip: use-named-expression
        """
        Send to the same textual channel, the current music playing in this Guild

        :param message: discord.Message
        """
        guild: discord.Guild = message.guild or message.author.guild

        info = Player.get(guild).get_now_played()

        if info:
            await message.channel.send(f'titre: {info[0]}.\nLien: {info[1]}')
        else:
            await message.channel.send("I'm not playing song")



import logging

from rizthme.models.threads import Player


def init(client: "Client"):

    @client.event
    async def on_ready():
        """
        this event run after the client is ready, the token is valid.

        warning: this event is not lit "on_connect".
        It's used to set up differents settings and variables.
        """
        # Setup Players Guild's thread
        Player.setup_music_queue(client)
        logging.info('Client Ready')



import discord


def init(client: "Client"):

    @client.command(["resume", "start"])
    async def resume(message: discord.Message):  # sourcery skip: use-named-expression
        """
        Pause the music of your current Guild

        :param message: discord.Message
        """
        guild: discord.Guild = message.guild or message.author.guild

        # Retrieves the bot's VoiceClient, if there is already one present in the server.
        voice_client: discord.VoiceClient = discord.utils.get(client.voice_clients, guild=guild)
        if not voice_client:
            return

        if voice_client.is_paused():
            voice_client.resume()
            await message.channel.send("Music resumed.")



import logging
from rizthme.setting.config import TOKEN


def init(client: "Client"):

    @client.event
    async def on_disconnect():
        """
        this event run after the server disconnect the client.

        It's used to relogin CLIENT.
        """
        logging.info('Client down!')
        await client.login(TOKEN)
        logging.info('Client up!')



import asyncio
import logging
import discord

from random import randint
from threading import Thread, Semaphore
from typing import Tuple, Optional, Dict

from rizthme.exception import DuplicateGuildPlayerThreadError, BadLinkError
from rizthme.models.musics import SimpleMusic, Playable
from rizthme.models.music_queue import MusicQueue
from rizthme.models.mode import MODE
from rizthme.models.factory import AudioFactory


class Player(Thread):
    _guild_thread: Dict[discord.Guild, "Player"] = {}

    @classmethod
    def get(cls, guild: discord.Guild) -> "Player":
        """
        Get the player of the guild
        :param guild: discord.Guild
        :return: Player
        """
        return cls._guild_thread[guild]

    @classmethod
    def setup_music_queue(cls, client: discord.Client):
        """
        Create a thread by discord.Guild where the CLIENT is present
        """
        AudioFactory.client = client
        for guild in client.guilds:
            # Starting a Player per guild
            try:
                Player(client, guild).start()
            except DuplicateGuildPlayerThreadError:
                logging.warning(f"Several Player is trying to be create for {guild.name}")

    @classmethod
    def set_voice_client(cls, guild: discord.Guild, voice_client: discord.VoiceClient):
        """
        Set to the appropriate thread, the voice client needed to play song
        :param guild: discord.Guild, Guild wanted to set the voice client
        :param voice_client: discord.VoiceClient
        """
        cls.get(guild)._voice_client = voice_client

    @classmethod
    def add_music(cls, message: discord.Message):
        """
        Add a music in the queue of the appropriate Guild
        :param message: discord.Message
        """
        try:
            music = AudioFactory.create_playable(message)
            guild: discord.Guild = message.guild if message.guild is not None else message.author.guild
            cls.get(guild)._add_queue(music)
        except BadLinkError as e:
            logging.warning(str(e))

    def __init__(self, client: discord.Client, guild: discord.Guild):
        super().__init__()
        self._guild: discord.Guild = guild
        if self._guild in self._guild_thread:
            logging.critical('PlayerThread created 2 time for 1 Guild')
            raise DuplicateGuildPlayerThreadError(self._guild)
        self._guild_thread[guild] = self
        self._voice_client: Optional[discord.VoiceClient] = None
        self._currently_playing_music: Optional[SimpleMusic] = None
        self._semaphore_is_playing: Semaphore = Semaphore(0)
        self._semaphore_queue: Semaphore = Semaphore(0)
        self._queue = MusicQueue(self._semaphore_queue)
        self._mode = MODE.NORMAL
        self._running = True
        self._client = client

    def delete_thread(self):
        """
        Delete the thread
        """
        self._running = False
        self._semaphore_is_playing.release()
        self._semaphore_queue.release()
        self._guild_thread.pop(self._guild)

    def run(self):
        while self._running:
            # Wait a music in the queue
            self._semaphore_queue.acquire()
            if not self._running:  # If the player is not running, stop the thread
                break
            # Get the music to play
            index = randint(0, len(self._queue) - 1) if self._mode == MODE.SHUFFLE else 0
            self._currently_playing_music: SimpleMusic = self._queue[index]

            # Play the music
            self._currently_playing_music.play(self._voice_client, after=lambda _: self._prepare_the_next_song())

            # Wait the music.play callback for continue
            self._semaphore_is_playing.acquire()

    def _prepare_the_next_song(self):
        """
        This private method is here to release the @semaphore_is_playing
        and used for the callback function after a song playing.
        """
        if self._mode == MODE.NORMAL:
            if self._queue:
                self._queue.pop(0)
        else:
            self._semaphore_queue.release()

        self._semaphore_is_playing.release()
        # Set the music currently playing to None if the queue is empty
        if not self._queue:
            voice_client = self._voice_client or discord.utils.get(self._client.voice_clients, channel=self._guild)
            if voice_client is not None:
                asyncio.run_coroutine_threadsafe(voice_client.disconnect(force=True), self._client.loop)
                self._currently_playing_music = None

    def get_now_played(self) -> Optional[Tuple[str, str]]:
        """
        Get a Tuple of the current music playing

        Metadata -> Tuple[ < Song Title >, < Song URL > ]

        :return: Tuple[str, str] or None
        """
        if self._currently_playing_music:
            return self._currently_playing_music.get_title(), self._currently_playing_music.get_url()

    def _add_queue(self, audio_item: Playable):
        """
        Add a playlist in the queue of the appropriate Guild
        :param playlist: Playlist
        """
        self._queue.add_music(audio_item)

    def clear_queue(self):
        """
        Clear the music queue of the Guild
        (used for the stop command)
        """
        self._semaphore_queue = Semaphore(0)
        self._queue = MusicQueue(self._semaphore_queue)
        self._mode = MODE.NORMAL
        if self._voice_client.is_paused():
            self._voice_client.resume()
        if self._voice_client and self._voice_client.is_playing():
            self._currently_playing_music.stop(self._voice_client)
        self._currently_playing_music = None

    def set_mode(self, mode: MODE):
        """
        Set the loop mode of the music player
        :param mode: MODE
        """
        self._mode = mode



import logging
import discord
from rizthme.models.threads import Player


def init(client: "Client"):

    @client.event
    async def on_guild_remove(guild: discord.Guild):
        """
        When a guild is removed, remove the music players linked to it.

        :param guild: The guild that was removed.
        :type guild: discord.Guild
        """
        logging.info(f'{guild.name} has been removed')
        Player.get(guild).delete_thread()



import discord
import pytube

from pytube.helpers import DeferredGeneratorList
from rizthme.models.musics.playlist import Playlist

from rizthme.models.musics.youtube.yt_music import YTMusic


class YTPlaylist(Playlist):

    def __init__(self, url: str, channel: discord.TextChannel):
        Playlist.__init__(self, url, channel)
        self.plt: pytube.Playlist = pytube.Playlist(url)
        self._title = self.plt.title

    def get_url(self) -> str:
        """
        :return: the url of the playlist
        """
        return self._url

    def _music_generator(self) -> str:
        """
        Generator that yield the music of the playlist
        :return: a music of the playlist
        """
        for youtube in self.plt.videos:
            yield YTMusic(youtube, self._channel)

    def get_list_music(self) -> DeferredGeneratorList:
        """
        :return: the list of music in the playlist
        """
        return DeferredGeneratorList(self._music_generator())

    def get_title(self) -> str:
        """
        :return: the title of the playlist
        """
        return self._title



from abc import ABC

from typing import Iterable

from rizthme.models.musics.audio_item import AudioItem


class Playlist(AudioItem, ABC):

    def get_list_music(self) -> Iterable["Music"]:
        """
        Get list of music from playlist
        """
        pass



from typing import List
from threading import Semaphore, Lock
from multipledispatch import dispatch

from rizthme.models.threads import playlist_queue_add, music_queue_add
from rizthme.models.musics import SimpleMusic, Playlist


class MusicQueue(List[SimpleMusic]):

    def __init__(self, semaphore: Semaphore, *args):
        super().__init__(*args)
        self._queue_semaphore = semaphore
        self._is_adding_lock = Lock()

    @dispatch(SimpleMusic)
    def add_music(self, music: SimpleMusic):
        # if the message is a valid song
        if music.is_valid(send_message=True):
            music_queue_add(self, music, self._queue_semaphore, self._is_adding_lock)

    @dispatch(Playlist)
    def add_music(self, playlist: Playlist):
        playlist.send("Loading playlist... (this may take a while)")
        playlist_queue_add(self, playlist, self._queue_semaphore, self._is_adding_lock)



from rizthme.models.musics import SimpleMusic
from threading import Semaphore, Lock
from tmktthreader import Threader
from typing import List


@Threader
def music_queue_add(queue: List[SimpleMusic], music, queue_semaphore: Semaphore, is_adding_lock: Lock):
    is_adding_lock.acquire()
    queue.append(music)
    queue_semaphore.release()
    music.send(f'Music: "{music.get_title()}", has been added')
    is_adding_lock.release()



import logging

import discord

from pytube import YouTube
from typing import Callable
from multipledispatch import dispatch

from rizthme.models.musics.simple_music import SimpleMusic


class YTMusic(SimpleMusic):
    """
    Class for handling music from YouTube.
    """

    def __init__(self, client: "Client", url: str | YouTube, channel: discord.TextChannel):
        SimpleMusic.__init__(self, client, url, channel)
        if isinstance(url, str):
            self.ytb: YouTube = YouTube(url)
        else:
            self.ytb: YouTube = url
        self._stream_url: str = self.ytb.streams.filter(only_audio=True).order_by('abr').desc().first().url

    def is_valid(self, send_message: bool = False) -> bool:
        """
        Check if the Music instance is correctly playable
        :param send_message: bool, Optional argument for send error message in message's channel with the error stack
        :return: True if the music is valid and playable
        """
        return self._is_url_valid(send_message) and self._is_valid_stream(send_message)

    def _is_url_valid(self, send_message: bool = False) -> bool:
        """
        Check the user's input url, and check if is it empty
        :param send_message: bool, Optional argument for send error message in message's channel with the error stack
        :return: True if the original_url isn't empty
        """
        valid: bool = self._url != ""
        if send_message and not valid:
            logging.warning("!play: Dont valid music")
            self.send("Something's wrong, try to use !play like this: \n !play [YouTube's Link]")
        return valid

    def _is_valid_stream(self, send_message: bool = False) -> bool:
        """
        Check if the audio stream from PyTube library is correct and usable
        :param send_message: bool, Optional argument for send error message in message's channel with the error stack
        :return: True if the stream is usable
        """
        valid: bool = self._stream_url is not None
        if send_message and not valid:
            logging.critical("!play: Dont valid music")
            self.send("Something's wrong, YouTube link is unfunctionnal")
        return valid

    def get_title(self) -> str:
        """
        :return: Title of the YouTube's video
        """
        return str(self.ytb.title)

    def get_url(self) -> str:
        """
        :return: user's input URL
        """
        return str(self.ytb.watch_url)

    def get_audio_source(self) -> discord.FFmpegPCMAudio:
        """
        Create a discord.FFmpegPCMAudio from the PyTube.Stream gotten in __init__

        The @before_options is here for reconnected automaticaly the voice_client to the stream
        :return: FFmpegPCMAudio usable like AudioSource for discord.VoiceClient
        """
        before_options = " -reconnect 1 -reconnect_streamed 1 -reconnect_delay_max 5"
        try:
            return discord.FFmpegPCMAudio(self._stream_url, before_options=before_options)
        except Exception as e:
            logging.critical(f'When FFmpegPCMAudio created:\n{e}')
            self.send('Error with the YouTube API function')

    def get_duration(self) -> int:
        """
        :return: duration of the YouTube's video in seconds
        """
        return self.ytb.length

    def play(self, voice_client: discord.VoiceClient, after: Callable = None) -> None:
        """
        Play the music with the voice_client
        :param voice_client: discord.VoiceClient, voice_client to play the music
        :param after: Callable, Optional argument for call after the music is played
        """
        voice_client.play(self.get_audio_source(), after=after)

    def stop(self, voice_client: discord.VoiceClient) -> None:
        """
        Stop the music with the voice_client
        :param voice_client: discord.VoiceClient, voice_client to stop the music
        """
        voice_client.stop()



import discord


def init(client: "Client"):

    @client.command(["skip", "next"])
    async def skip(message: discord.Message):
        """
        pass the music of your current Guild

        :param client: Client discord
        :param message: discord.Message
        """
        voice_client: discord.VoiceClient = discord.utils.get(client.voice_clients, guild=message.guild)
        if not voice_client:
            return
        if voice_client.is_paused():
            voice_client.resume()
        if voice_client.is_playing():
            voice_client.stop()



import os
import subprocess
import discord


def init(client: "Client"):

    @client.command(["kill"])
    async def kill(message: discord.Message):
        """
        This command is for stop the CLIENT,

        If someone wants to use this command,
        his discord user's id need to be in the hard-coded id list
        :param message: discord.Message
        """
        list_admin = [290227829558345728]
        if message.author.id in list_admin:
            await message.reply("SIR, YES SIR !")

            for voice_client in client.voice_clients:
                await voice_client.disconnect()

            # This is a command to kill the current process.
            # Because discord.py API don't make it after disconnected,
            # I need to kill the current process to stop the program.
            # I know that's not a good solution. Contact tamikata#2214 if you have another solution.
            subprocess.run(f'kill -9 {os.getpid()}', shell=True)



import discord
from rizthme.models.threads import Player
from rizthme.models.mode import MODE


def init(client: "Client"):

    @client.command(["loop"])
    async def loop(message: discord.Message):
        """
        Setting up the loop mode to the music player in the guild.
        """
        guild: discord.Guild = message.guild or message.author.guild

        Player.get(guild).set_mode(MODE.LOOP)
        await message.channel.send('Mode: Loop activeted !')



from rizthme.models.musics.playable import Playable
from rizthme.models.musics.playlist import Playlist
from rizthme.models.musics.audio_item import AudioItem
from rizthme.models.musics.simple_music import SimpleMusic
from rizthme.models.musics.youtube import *



import asyncio
from abc import ABC, abstractmethod

import discord


class AudioItem(ABC):

    def __init__(self, client, url, channel: discord.TextChannel):
        self.client = client
        self._url = url
        self._channel = channel

    @abstractmethod
    def get_title(self) -> str:
        """
        Get title
        """
        pass

    @abstractmethod
    def get_url(self) -> str:
        """
        Get url
        """
        pass

    def send(self, message: str):
        """
        Send message to channel
        """
        asyncio.run_coroutine_threadsafe(self._channel.send(message), self.client.loop)



import logging
import discord

from rizthme.models.threads import Player


def init(client: "Client"):

    @client.event
    async def on_guild_join(guild: discord.Guild):
        """
        this event is called when a new guild is joined or created by the CLIENT.

        :param guild: The guild that was joined.
        """
        logging.info(f"Joined guild {guild.name}")
        Player(client, guild).start()



import discord

from rizthme.models.threads import Player


def init(client: "Client"):

    @client.command(["stop"])
    async def stop(message: discord.Message):
        """
        Stop the music of your current Guild
        :param message: discord.Message
        """
        guild = message.guild or message.author.guild

        p = Player.get(guild)
        p.clear_queue()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rizthme',
 'rizthme.commands',
 'rizthme.events',
 'rizthme.models',
 'rizthme.models.factory',
 'rizthme.models.musics',
 'rizthme.models.musics.youtube',
 'rizthme.models.threads',
 'rizthme.setting']

package_data = \
{'': ['*']}

install_requires = \
['discord>=2.1.0,<3.0.0',
 'ffmpeg>=1.4,<2.0',
 'multipledispatch>=0.6.0,<0.7.0',
 'pynacl>=1.5.0,<2.0.0',
 'pytube>=12.1.2,<13.0.0',
 'tmktthreader>=1.0.1,<2.0.0']

setup_kwargs = {
    'name': 'rizthme',
    'version': '2.1.0',
    'description': 'Discord Bot',
    'long_description': '# Rizthme\n\n## Requierement\n\nFirst step is to add in your environments variables,\na variables named "TOKEN" with in value, your discord application token.\n\ndo this with:\n\n```console\nfoo@bar:~$ export TOKEN=<your token>\n```\n\nnext, check that the ffmpeg module is installed on your device.\n\n```console\nfoo@bar:~$ ffmpeg -version\n```\n\nif not\n\n```console\nfoo@bar:~$ sudo apt install ffmpeg \n# or equivalent depending on your system\n```\n\n## To launch the discord client\n\n```console\nfoo@bar:~$ python3 client.py\n```\n\n### Automatical script launching\n\n#### For Linux user\n\nFor use automatically your virtual environment. (The virtual environment name need to be "venv/")\n\n```console\nfoo@bar:~$ source entrypoint.sh\n```\n\nif you don\'t want to use a virtual environment. juste use the script like this:\n\n```console\nfoo@bar:~$ bash entrypoint.sh\n```\nor \n\n```console\nfoo@bar:~$ sh entrypoint.sh\n```\n\n#### For Windows user\n\n> Not sure yet\n\nyou can do like this if you realy want to use that in Windows system\n\n```\nC:RizThme\\> entrypoint.bat\n```\n',
    'author': 'ladettanguy',
    'author_email': 'sti2dlab.ladettanguy@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



import asyncio

import discord
import logging


def init(client: "Client"):

    @client.event
    async def on_message(message: discord.Message):
        """
        This event is called when a message is sent in a channel.

        :param message: The message that was sent.
        """
        # Ignore bot messages
        if message.author.bot:
            return

        # Ignore messages without prefix
        if not message.content.startswith(client.PREFIX):
            return

        # Split message into command and arguments
        cmd_line = message.content.split(' ')
        # Get command
        cmd: str = cmd_line[0]
        cmd = cmd.replace(client.PREFIX, '')

        # if command is in commands list
        if cmd in client.commands:
            logging.info(f"command: {cmd}, by {message.author} in {message.guild} on {message.channel} channel")
            func = client.commands[cmd]
            # Call the appropriate for the command wanted and pass the discord.Message in parameters
            asyncio.run_coroutine_threadsafe(func(message), client.loop)



import discord


class RizthmeException(Exception):
    pass


class DuplicateGuildPlayerThreadError(RizthmeException):
    def __init__(self, guild: discord.Guild):
        self.guild = guild

    def __str__(self):
        return f"You try to create a 2nd Player for the guild {self.guild}"


class BadLinkError(RizthmeException):
    def __init__(self, link: str):
        self.link = link

    def __str__(self):
        return f"Bad link: {self.link}"



from rizthme.models.musics import Playlist, SimpleMusic
from threading import Semaphore, Lock
from tmktthreader import Threader
from typing import List


@Threader
def playlist_queue_add(queue: List[SimpleMusic], playlist: Playlist, queue_semaphore: Semaphore, is_adding_lock: Lock):
    is_adding_lock.acquire()
    # Add all music from the playlist to the queue
    for music in playlist.get_list_music():  # get_list_music() return a DeferredGeneratorList.
        # If the playable is valid, so it can be added to the queue.
        if music.is_valid(send_message=True):
            queue.append(music)
            queue_semaphore.release()
    playlist.send(f"Playlist: {playlist.get_title()} has been added")
    is_adding_lock.release()



import os
import discord
import importlib.util
from glob import glob
from typing import Iterable
dir_name, name_file = os.path.split(__file__)


def import_module_from_file(full_path_to_module):
    # Get module name and path from full path
    module_dir, module_file = os.path.split(full_path_to_module)
    module_name, module_ext = os.path.splitext(module_file)

    # Get module "spec" from filename
    spec = importlib.util.spec_from_file_location(module_name, full_path_to_module)

    return spec.loader.load_module()


class Client(discord.Client):

    def __init__(self, prefix="!"):
        self.commands = {}
        self.PREFIX = prefix
        discord.Client.__init__(self, intents=discord.Intents.all())

    def load(self):
        """
        Initialise all commands and events
        """
        self._init_directory("events")
        self._init_directory("commands")

    def _init_directory(self, directory: str):
        """
        Initialise all events
        """
        path = os.path.join(dir_name, directory)
        list_module = [
            import_module_from_file(path) for path in glob(os.path.abspath(path) + '/*.py')
            if not "__" in path
        ]
        for module in list_module:
            init = getattr(module, 'init')
            init(self)

    def command(self, alias: Iterable[str]):
        """
        add a function to all aliases.

        This function is call when a message started with the alias is sent on a discord textual channel
        :param alias: List[str], list of command, callable with a discord.Message (Without PREFIX)
        """
        def decorator(f):
            """
            :param f: Callable, function to call when the alias is sent in a message.
            """
            for name in alias:
                if name in self.commands:
                    raise ValueError(f"Command {name} already exists")
                self.commands[name] = f
            return f
        return decorator



import discord

from rizthme.models.threads import Player
from rizthme.models.mode import MODE


def init(client: "Client"):

    @client.command(["shuffle"])
    async def shuffle(message: discord.Message):
        """
        Setting up the shuffle mode to the music player in the guild.
        """
        guild: discord.Guild = message.guild or message.author.guild

        Player.get(guild).set_mode(MODE.SHUFFLE)
        await message.channel.send('Mode: Shuffle activeted !')
